'
' Write and read a 25LC256 SPI eeprom using the microcontroller's SPI peripheral
'
    Device = 24HJ128GP502        
    Declare Xtal = 32
    
    Declare Hserial_Baud = 38400
    Declare Hrsout1_Pin = PORTB.7                       ' Select which pin is to be used for TX with USART1
    Declare Hrsin1_Pin = PORTB.6                        ' Select which pin is to be used for RX with USART1
    
    Include "SPI_24.inc"                                ' Load the PIC24 SPI macros into the program
    
    Dim wAddress As Word                                ' Address within the eeprom
    Dim Byteout As Byte                                 ' The byte written to the eeprom
    Dim Bytein As Byte                                  ' The byte read from the eeprom

    Symbol CS_Pin = PORTB.0                             ' Connects to the eeprom's CS pin

'--------------------------------------------------------------------------------------
' Initialise the SPI interface
' Input     : None
' Output    : None
' Notes     : Also configures PPS for the SPI1 peripheral
'           : Final SPI speed (in KHz) = ((fOSC / 2) / 1000) / (Secondary Prescale * Primary Prescale)
'
Proc SPI_Init()
'
' Setup the SPI as master Mode 0:0
' When operating at 32MHz, the prescaler values below will operate the SPI clock at 8MHz:
'   ((32000000 / 2) / 1000) / (2 * 1) = 8000KHz
'
    SPI_Open(cSPI_MODE0 & cSEC_PRE_2_1 & cPRI_PRE_1_1, cSPI_ON & cSPI_IDLE_ON & cSPI_RX_OVFLOW_CLR)
'
' Setup the PPS pins for SPI
'
    PPS_Output(cOut_Pin_RP14, cOut_Fn_SCK1)             ' To SPI peripheral's CLK pin
    PPS_Output(cOut_Pin_RP13, cOut_Fn_SDO1)             ' To SPI peripheral's SDO pin
    PPS_Input(cIn_Pin_RP12, cIn_Fn_SDI1)                ' From SPI peripheral's SDI pin
    High PORTB.14                                       ' Make the SCK pin output high
    High PORTB.13                                       ' Make the SDO pin output high
    Input PORTB.12                                      ' Make the SDI pin an input
    High CS_Pin                                         ' Make the CS pin output high    
EndProc

'--------------------------------------------------------------------------------------
Main:    
    SPI_Init()                                          ' Configure the SPI interface 
'
' Write to the SPI eeprom
'
    Byteout = 1                                         ' Initial write value
    wAddress = $00                                      ' Reset the address
    Repeat                                              ' Create a loop
        Clear CS_Pin                                    ' Enable the SPI interface
        SPI_Write($06)                                  ' Send the Write Enable command
        Set CS_Pin                                      ' Disable the SPI interface to execute the command
               
        Clear CS_Pin                                    ' Enable the SPI interface
        SPI_Write($02)                                  ' Send the Write command
        SPI_Write(wAddress.Byte1)                       ' \ Send the 16-bit address (MSB first)
        SPI_Write(wAddress.Byte0)                       ' /
        SPI_Write(Byteout)                              ' Send the data to the eeprom
        Set CS_Pin                                      ' Disable the SPI interface

        DelayMS 5                                       ' Give time for the data to be written within the eeprom   
        Inc wAddress                                    ' Increment the address
        Inc Byteout                                     ' \
    Until wAddress >= 9                                 ' / Stop after 10 writes
'
' Read from the SPI eeprom
'
    wAddress = $00                                      ' Reset the address
    Repeat                                              ' Create a loop
        Clear CS_Pin                                    ' Enable the SPI interface
        SPI_Write($03)                                  ' Send the Read command
        SPI_Write(wAddress.Byte1)                       ' \ Send the 16-bit address (MSB first)
        SPI_Write(wAddress.Byte0)                       ' /
        Bytein = SPI_Read()                             ' Read data from the eeprom
        Set CS_Pin                                      ' Disable the SPI interface        
        HRSOut IHex4 wAddress, " : ", IHex2 Bytein, 13  ' Serially transmit the address and eeprom contents        
        Inc wAddress                                    ' \
    Until wAddress >= 9                                 ' / Stop after 10 reads 
   
