'
' Write and read a 25LC256 SPI eeprom using the microcontroller's SPI peripheral
'
    Device = 24FJ64GA002        
    Declare Xtal = 32
    Declare Hserial_Baud = 9600                         ' USART1 baud rate
    Declare Hrsout1_Pin = PORTB.14                      ' Select the pin used for TX with USART1
    
    Include "SPI_24.inc"                                ' Load the 24-bit SPI macros into the program
    
    Dim wAddress As Word                                ' Address within the eeprom
    Dim Byteout As Byte                                 ' The byte written to the eeprom
    Dim Bytein As Byte                                  ' The byte read from the eeprom

    Symbol CS_Pin = PORTB.0                             ' Connects to the eeprom's CS pin

'--------------------------------------------------------------------------------------
    GoTo Main                                           ' Jump over the subroutine
'--------------------------------------------------------------------------------------
' Initialise the SPI interface
' Input     : None
' Output    : None
' Notes     : Also configures PPS for the SPI1 peripheral
'           : Final SPI speed (in KHz) = ((fOSC / 2) / 1000) / (Secondary Prescale * Primary Prescale)
'
Proc SPI_Init()
'
' Setup the SPI as master Mode 0:0
' When operating at 32MHz, the prescaler values below will operate the SPI clock at 8MHz:
'   ((32000000 / 2) / 1000) / (2 * 1) = 8000KHz
'
    SPI_Open(cSPI_MODE0 & cSEC_PRE_2_1 & cPRI_PRE_1_1, cSPI_ON & cSPI_IDLE_ON & cSPI_RX_OVFLOW_CLR)
'
' Setup the PPS pins for SPI
'
    PPS_Output(cOut_Pin_RP4, cOut_Fn_SCK1OUT)           ' To SPI peripheral's CLK pin
    PPS_Output(cOut_Pin_RP3, cOut_Fn_SDO1)              ' To SPI peripheral's SDO pin
    PPS_Input(cIn_Pin_RP2, cIn_Fn_SDI1)                 ' From SPI peripheral's SDI pin
    High PORTB.4                                        ' Make the SCK pin output high
    High PORTB.3                                        ' Make the SDO pin output high
    Input PORTB.2                                       ' Make the SDI pin an input
    High CS_Pin                                         ' Make the CS pin output high    
EndProc

'--------------------------------------------------------------------------------------
Main:  
$if (_type = _PIC24F)                                   ' Are we using a PIC24F device?
    CLKDIV = 0                                          ' Yes. So CPU peripheral clock ratio set to 1:1  
    Write_OSCCONH(%00010000)                            ' Enable PLL
$endif

    PPS_Output(cOut_Pin_RP14, cOut_Fn_U1TX)             ' Make PPS Pin RP14 U1TX
      
    SPI_Init()                                          ' Configure the SPI interface 
'
' Write to the SPI eeprom
'
    
        
    Byteout = 0                                         ' Initial write value
    wAddress = $00                                      ' Reset the address
    Repeat                                              ' Create a loop       
        Clear CS_Pin                                    ' Enable the SPI interface
        SPI_Write($06)                                  ' Send the Write Enable command
        Set CS_Pin                                      ' Disable the SPI interface to execute the command       

        Clear CS_Pin                                    ' Enable the SPI interface
        SPI_Write($02)                                  ' Send the Write command
        SPI_Write(wAddress.Byte1)                       ' \ Send the 16-bit address (MSB first)
        SPI_Write(wAddress.Byte0)                       ' /
        SPI_Write(Byteout)                              ' Send the data to the eeprom
        Set CS_Pin                                      ' Disable the SPI interface

        DelayMS 5                                       ' Give time for the data to be written within the eeprom   
        Inc wAddress                                    ' Increment the address
        'Inc Byteout                                     ' \
    Until wAddress >= 10                                 ' / Stop after 10 writes
'
' Read from the SPI eeprom
'
    wAddress = $00                                      ' Reset the address
    Repeat                                              ' Create a loop
        Clear CS_Pin                                    ' Enable the SPI interface
        SPI_Write($03)                                  ' Send the Read command
        SPI_Write(wAddress.Byte1)                       ' \ Send the 16-bit address (MSB first)
        SPI_Write(wAddress.Byte0)                       ' /
        Bytein = SPI_Read()                             ' Read data from the eeprom
        Set CS_Pin                                      ' Disable the SPI interface
        
        HRSOut IHex4 wAddress, " : ", IHex2 Bytein, 13  ' Serially transmit the address and eeprom contents        
        Inc wAddress                                    ' \
    Until wAddress >= 10                                 ' / Stop after 10 reads 
    Stop
        
'--------------------------------------------------------------------------------------
$if _device = _24FJ64GA002  
' For internal 8MHz oscillator with PLL
' OSC pins are general purpose I/O
'  
    Config Config1 = JTAGEN_OFF, GCP_OFF, GWRP_OFF, BKBUG_OFF, COE_OFF, ICS_PGx1, FWDTEN_OFF, WINDIS_OFF, FWPSA_PR128, WDTPOST_PS256
    Config Config2 = IOL1WAY_OFF, COE_OFF, IESO_OFF, FNOSC_FRCPLL, FCKSM_CSDCMD, OSCIOFNC_ON, POSCMOD_NONE 
$endif  ' _24FJ64GA002    
