'
' Write and read a 25LC256 SPI eeprom using software SPI routines
'
    Device = 24FJ64GA002        
    Declare Xtal = 32
    Declare Hserial_Baud = 9600                         ' USART1 baud rate
    Declare Hrsout1_Pin = PORTB.14                      ' Select the pin used for TX with USART1
    
    Include "PPS.inc"                                   ' Load the 24-bit PPS macros into the program
'
' Setup the pins for the software SPI routines
'    
    $define SPI_SDOPin PORTB.3                          ' SPI DO to eeprom's DI pin
    $define SPI_SDIPin PORTB.2                          ' SPI DI to eeprom's DO pin
    $define SPI_CLKPin PORTB.4                          ' SPI CLK pin
    $define SPI_CSPin PORTB.0                           ' SPI CS pin (active low)

    Include "Software_SPI.inc"                          ' Load the software SPI macros into the program
    
    Dim wAddress As Word                                ' Address within the eeprom
    Dim Byteout As Byte                                 ' The byte written to the eeprom
    Dim Bytein As Byte                                  ' The byte read from the eeprom

'--------------------------------------------------------------------------------------
    GoTo Main                                           ' Jump over the subroutine

'--------------------------------------------------------------------------------------
Main:  
$if (_type = _PIC24F)                                   ' Are we using a PIC24F device?
    CLKDIV = 0                                          ' Yes. So CPU peripheral clock ratio set to 1:1  
    Write_OSCCONH(%00010000)                            ' Enable PLL
$endif

    PPS_Output(cOut_Pin_RP14, cOut_Fn_U1TX)             ' Make PPS Pin RP14 U1TX
'
' Write to the SPI eeprom
'
    Byteout = 1                                         ' Initial write value
    wAddress = $00                                      ' Reset the address
    Repeat                                              ' Create a loop
        Clear SPI_CSPin                                 ' Enable the SPI interface
        SPI_Write($06)                                  ' Send the Write Enable command
        Set SPI_CSPin                                   ' Disable the SPI interface to execute the command
               
        Clear SPI_CSPin                                 ' Enable the SPI interface
        SPI_Write($02)                                  ' Send the Write command
        SPI_Write16(wAddress)                           ' Send the 16-bit address (MSB first) 
        SPI_Write(Byteout)                              ' Send the data to the eeprom
        Set SPI_CSPin                                   ' Disable the SPI interface

        DelayMS 5                                       ' Give time for the data to be written within the eeprom   
        Inc wAddress                                    ' Increment the address
        Inc Byteout                                     ' \
    Until wAddress >= 9                                 ' / Stop after 10 writes
'
' Read from the SPI eeprom
'
    wAddress = $00                                      ' Reset the address
    Repeat                                              ' Create a loop
        Clear SPI_CSPin                                 ' Enable the SPI interface
        SPI_Write($03)                                 ' Send the Read command
        SPI_Write16(wAddress)                           ' Send the 16-bit address (MSB first)
        Bytein = SPI_Read()                            ' Read data from the eeprom
        Set SPI_CSPin                                   ' Disable the SPI interface
        
        HRSOut IHex4 wAddress, " : ", IHex2 Bytein, 13  ' Serially transmit the address and eeprom contents        
        Inc wAddress                                    ' \
    Until wAddress >= 9                                 ' / Stop after 10 reads 
    Stop
        
'--------------------------------------------------------------------------------------
$if _device = _24FJ64GA002  
' For internal 8MHz oscillator with PLL
' OSC pins are general purpose I/O
'  
    Config Config1 = JTAGEN_OFF, GCP_OFF, GWRP_OFF, BKBUG_OFF, COE_OFF, ICS_PGx1, FWDTEN_OFF, WINDIS_OFF, FWPSA_PR128, WDTPOST_PS256
    Config Config2 = IOL1WAY_OFF, COE_OFF, IESO_OFF, FNOSC_FRCPLL, FCKSM_CSDCMD, OSCIOFNC_ON, POSCMOD_NONE 
$endif  ' _24FJ64GA002 
   
