'
' Software SPI read and write
'
'
' SPI Declarations
'
$ifndef SPI_SDOPin
    $define SPI_SDOPin PORTB.13                  ' SPI DO to peripheral DI pin
$endif
$ifndef SPI_SDIPin
    $define SPI_SDIPin PORTB.12                  ' SPI DI to peripheral DO pin
$endif
$ifndef SPI_CLKPin
    $define SPI_CLKPin PORTB.11                  ' SPI CLK pin
$endif
$ifndef SPI_CSPin
    $define SPI_CSPin PORTB.0                   ' SPI CS pin (active low)
$endif

'----------------------------------------------------------------------------    
    GoTo _Software_SPI_Main_
'-----------------------------------------------------------------------------------
' Write a single Byte (8-bits) to the SPI bus.
' Input     : WREG0 holds the byte to transmit to the SPI interface
' Output    : None
' Notes     : At Fosc 140MHz, the SPI interace will operate at approx 10MHz
'
Proc SPI_Write(pDataOut As WREG0.Byte0)
    WREG1 = 8                           ' Single byte SPI loop
    Repeat
        SPI_SDOPin = WREG0.7            ' Put the current outgoing bit on SPI_SDOpin
        WREG0 = WREG0 << 1              ' Shift the next bit into MSB
        Set SPI_CLKPin                  ' Set SPI_CLKpin high
        'DelayCS 10
        Clear SPI_CLKPin                ' Pull SPI_CLKpin low
        Dec WREG1
     Until SRbits_Z = 1                 ' Do 8-bits
EndProc
'-----------------------------------------------------------------------------------
' Read a single Byte from the SPI bus.
' Input     : None
' Output    : WREG0 holds any data read from the SPI interface
' Notes     : At Fosc 140MHz, the SPI interace will operate at approx 10MHz
'
'$define SPI_Read8() SPI_Read()

Proc SPI_Read(), WREG0.Byte
    WREG1 = 8                           ' Create a single byte loop
    Repeat
        WREG0 = WREG0 << 1              ' Shift the next bit into MSB
        Set SPI_CLKPin                  ' Set SPI_CLKpin high
        'DelayCS 10
        WREG0 = WREG0 | SPI_SDIPin      ' Capture the current bit on SPI_SDIpin
        Clear SPI_CLKPin                ' Pull SPI_CLKpin low
        Dec WREG1
    Until SRbits_Z = 1                  ' Do 8-bits
EndProc
'-----------------------------------------------------------------------------------
' Write 16-bits to the SPI bus (Most Significant Bit First).
' Input     : WREG0 holds the word to transmit to the SPI interface
' Output    : None
' Notes     : At Fosc 140MHz, the SPI interace will operate at approx 10MHz
'
Proc SPI_Write16(pDataOut As WREG0)
    WREG1 = 16                          ' 16-bit SPI loop
    Repeat
        SPI_SDOPin = WREG0.15           ' Put the current outgoing bit on SPI_SDOpin
        WREG0 = WREG0 << 1              ' Shift the next bit into MSB
        Set SPI_CLKPin                  ' Set SPI_CLKpin high
        'DelayCS 10
        Clear SPI_CLKPin                ' Pull SPI_CLKpin low
        Dec WREG1
     Until SRbits_Z = 1                 ' Do 16-bits
EndProc
'
'-----------------------------------------------------------------------------------
' Read 16-bits from the SPI bus (Most Signinficant Bit First)
' Input     : None
' Output    : WREG0 holds any data read from the SPI interface
' Notes     : At Fosc 140MHz, the SPI interace will operate at approx 10MHz
'
Proc SPI_Read16(), WREG0
    WREG1 = 16                          ' Create a 16-bitloop
    Repeat
        WREG0 = WREG0 << 1              ' Shift the next bit into MSB
        Set SPI_CLKPin                  ' Set SPI_CLKpin high
        'DelayCS 10
        WREG0 = WREG0 | SPI_SDIPin      ' Capture the current bit on SPI_SDIpin
        Clear SPI_CLKPin                ' Pull SPI_CLKpin low
        Dec WREG1
    Until SRbits_Z = 1                  ' Do 16-bits
EndProc
'-----------------------------------------------------------------------------------
' Setup the software SPI pins for input and output
'    
_Software_SPI_Main_:
    High SPI_SDOPin                     ' SPI DO to output high
    Input SPI_SDIPin                    ' SPI DI to input
    High SPI_CLKPin                     ' SPI CLK to output high
    High SPI_CSPin                      ' SPI CS to output high
   
    
